package typesampling

import scala.collection.mutable.{ArrayBuffer,HashMap,HashSet}
import fig.exec.Execution
import tea.Utils
import tea.Utils.{map,foreach,fmt,fmts,fmt1,fmt2,returnFirst,assertValid}
import tea.Utils.{track,begin_track,end_track,logs,dbgs}

import java.util.{Set,ArrayList,Random}
import java.io.File

// Context specifies the type of a tree (think of a type in programming languages)
trait Context
case class PrimitiveContext(value:Any) extends Context {
  override def toString = value.toString
}
object Empty { override def toString = "-" }
object PrimitiveContext {
  val empty = PrimitiveContext(Empty)
}

////////////////////////////////////////////////////////////

trait AbstractNode[ContextType, NodeType <: AbstractNode[ContextType, NodeType]] {
  def context : ContextType

  def foreachNode(f:NodeType=>Any) : Unit // (prefix traversal)
  def write(puts:String=>Any, indent:Int) : Unit
  def write(puts:String=>Any) : Unit = write(puts, 0)
  def log = write(logs("%s", _), 0)
}

trait AbstractPrimitiveNode[ContextType, NodeType <: AbstractNode[ContextType, NodeType]] {
  self:NodeType =>
  def value : AnyRef

  def foreachNode(f:NodeType=>Any) : Unit = f(this)
  override def toString = "("+context+" "+value+")"
  def write(puts:String=>Any, indent:Int) : Unit = puts(Misc.indentStr(indent) + toString)
}

trait AbstractCompositeNode[ContextType, NodeType <: AbstractNode[ContextType, NodeType]] {
  self:NodeType =>
  def left : NodeType
  def right : NodeType

  def foreachNode(f:NodeType=>Any) : Unit = { f(this); left.foreachNode(f); right.foreachNode(f) }
  override def toString = "("+context+" "+left+" "+right+")"
  def write(puts:String=>Any, indent:Int) : Unit = {
    puts(Misc.indentStr(indent) + "("+context)
    left.write(puts, indent+1)
    right.write(puts, indent+1)
    puts(Misc.indentStr(indent) + ")")
  }
}

////////////////////////////////////////////////////////////

// A node identifies a subtree.
// This is the generic representation which mediates domain-specific representations (Java, recipes)
// and the model-specific representations (PCFG, PTSG).
trait Node extends AbstractNode[Context, Node]
case class PrimitiveNode(context:Context, value:AnyRef) extends Node with AbstractPrimitiveNode[Context, Node]
case class CompositeNode(context:Context, left:Node, right:Node) extends Node with AbstractCompositeNode[Context, Node]
