package typesampling.pcfg

import typesampling._
import scala.collection.mutable.{ArrayBuffer,HashMap,HashSet}
import fig.exec.Execution
import tea.Utils
import tea.Utils.{map,foreach,fmt,fmts,fmt1,fmt2,returnFirst,assertValid}
import tea.Utils.{track,begin_track,end_track,logs,dbgs}

import java.util.{Set,ArrayList,Random}

case class RuleOutput[Context](left:Context, right:Context)

case class ContextPCFGLearner(opts:LearnOptions) extends PCFGLearner[Context] {
  def convertContext(context:Context) = context
}
case class StringPCFGLearner(opts:LearnOptions) extends PCFGLearner[String] {
  def convertContext(context:Context) = context.asInstanceOf[PrimitiveContext].value.asInstanceOf[String]
}

abstract class PCFGLearner[Context] extends Learner {
  type Example = Node
  def convertNode(node:Node) = node
  
  // Choices
  type Choice = String
  val C_LEX = "LEX"
  val C_RULE = "RULE"

  def ruleOutput(node:CompositeNode) = RuleOutput(convertContext(node.left.context), convertContext(node.right.context))

  override def log = {
  }

  class Params extends AbstractParams {
    val startCounts = new CounterMap[Context]
    val choiceCounts = new CounterMapCollection[Context,Choice]
    val lexCounts = new CounterMapCollection[Context,AnyRef]
    val ruleCounts = new CounterMapCollection[Context,RuleOutput[Context]]
    def write(maxPrint:Int, puts:String=>Any) = {
      startCounts.write(maxPrint, true, "START\t", puts)
      choiceCounts.write(maxPrint, maxPrint, true, "CHOICE\t", puts)
      lexCounts.write(maxPrint, maxPrint, true, "LEX\t", puts)
      ruleCounts.write(maxPrint, maxPrint, true, "RULE\t", puts)
    }
    def size = startCounts.numNonZero + choiceCounts.numNonZero + lexCounts.numNonZero + ruleCounts.numNonZero

    override def writeGrammar(puts:String=>Any) = {
      startCounts.keys.foreach { context:Context =>
        puts(fmts("(ROOT %s[0]) :: %s[0] ||| logProb:%s count:1", context, context, math.log(startCounts.getProb(context))))
      }
      // Note: Ignore choiceCounts because assume categories are disjoint
      lexCounts.keys.foreach { context:Context =>
        val map = lexCounts(context)
        map.keys.foreach { value:AnyRef =>
          puts(fmts("(%s %s) :: %s ||| logProb:%s count:1", context, value, value, math.log(map.getProb(value))))
        }
      }
      ruleCounts.keys.foreach { context:Context =>
        val map = ruleCounts(context)
        map.keys.foreach { out:RuleOutput[Context] =>
          puts(fmts("(%s %s[0] %s[1]) :: %s[0] %s[1] ||| logProb:%s count:1", context, out.left, out.right, out.left, out.right, math.log(map.getProb(out))))
        }
      }
    }
  }

  val params = new Params
  val avgParams = params

  class IterState extends super.IterState {
    override def run = false // Don't do anything
    def processExample(rootNode:Node) = { }
    def updateInferenceStats = { }
  }
  def newIterState = new IterState

  def convertContext(context:typesampling.Context) : Context

  override def learnInit = {
    super.learnInit
    trainExamples.foreach { rootNode:Node =>
      params.startCounts.incr(convertContext(rootNode.context), 1)
      rootNode.foreachNode { node:Node =>
        val context = convertContext(node.context)
        node match {
          case node:PrimitiveNode =>
            params.choiceCounts.incr(context, C_LEX, 1)
            params.lexCounts.incr(context, node.value, 1)
          case node:CompositeNode =>
            params.choiceCounts.incr(context, C_RULE, 1)
            params.ruleCounts.incr(context, ruleOutput(node), 1)
        }
      }
    }
  }

  def repCheck = { }

  def getInferenceStats(node:Node) = Array(0.0)

  def stateLogProb = {
    Utils.sum(trainExamples.size, { i:Int => logProb(trainExamples(i)) })
  }
}
