package typesampling

import scala.collection.mutable.{ArrayBuffer,HashMap,HashSet}
import fig.exec.Execution
import tea.Utils
import tea.Utils.{map,foreach,fmt,fmts,fmt1,fmt2,returnFirst,assertValid}
import tea.Utils.{track,begin_track,end_track,logs,dbgs,error}

import java.util.{Set,ArrayList,Random}
import java.io.File

class WordsOptions {
  import tea.OptionTypes._
}

/**
 * File format: each sentence (tree/node) is on a line.
 * The segmentation is encoded as follows:
 * abc d ef => (((a (b c)) d) (e f))
 */
class WordsDomain(options:WordsOptions) extends Domain {
  def fileExt = "txt"

  def write(node:Node, puts:String=>Any) = node.write(puts)

  // The context is a boolean indicating whether the words in the subtree below form a segment
  // (for evaluation purposes)
  def readNodes(path:String, maxNodes:Int) = {
    def combineWords(nodes:Array[Node], n:Int) : Node = {
      require (n > 0)
      if (n == 1) nodes(0)
      else CompositeNode(PrimitiveContext(false), combineWords(nodes, n-1), nodes(n-1))
    }
    def combineChars(word:String, n:Int) : Node = {
      require (n > 0)
      val node = PrimitiveNode(PrimitiveContext(word.size == 1), ""+word(n-1))
      if (n == 1) node
      else CompositeNode(PrimitiveContext(word.size == n), combineChars(word, n-1), node)
    }

    val nodes = new ArrayBuffer[Node]
    Utils.foreachLine(path, { line:String =>
      if (nodes.size < maxNodes) {
        val words = line.split("\\s+")
        //dbgs("%s", words.mkString("|||"))
        nodes += combineWords(words.map { word => combineChars(word, word.size) }, words.size)
        true
      }
      else
        false
    })
    nodes
  }
}
