package typesampling

import scala.collection.mutable.{ArrayBuffer,HashMap,HashSet}
import fig.exec.Execution
import tea.Utils
import tea.Utils.{map,foreach,fmt,fmts,fmt1,fmt2,returnFirst,assertValid}
import tea.Utils.{track,begin_track,end_track,logs,dbgs,error}

import java.util.{Set,ArrayList,Random}
import java.io.File


class ToyOptions {
  import tea.OptionTypes._
  @Option var numReplicates = 1
  @Option var testCase = 0
}

class ToyDomain(options:ToyOptions) extends Domain {
  def fileExt = "toy"

  def write(node:Node, puts:String=>Any) = node.write(puts)

  def q(c:Any, x:AnyRef) = PrimitiveNode(PrimitiveContext(c), x)
  def q(c:Any, left:Node, right:Node) = CompositeNode(PrimitiveContext(c), left, right)

  // Ignore path
  def readNodes(path:String, maxNodes:Int) = {
    def repeat[A](a: =>A, n:Int) : List[A] = if (n == 0) Nil else a :: repeat(a, n-1)

    options.testCase match {
      case 0 => q("A", "x") :: Nil
      case 1 => q("A", q("A", "x"), q("A", "y")) :: Nil
      case 2 => q("A", q("B", "x"), q("C", "y")) :: Nil
      case 3 => q("A", q("C", "x"), q("A", q("C", "x"), q("B", "y1"))) ::
                q("A", q("C", "x"), q("A", q("C", "x"), q("B", "y2"))) ::
                q("A", q("C", "x"), q("A", q("C", "x"), q("B", "y3"))) ::
                Nil
      case 4 =>
        // Desiderata: when a unique pattern shows up once, it should be decomposed
        // The more times it shows up, the more likely it should be one unit
        repeat(q("A", q("B", "x"), q("C", "y")), options.numReplicates)

      case 5 =>
        // Test case for refactoring
        q("A", q("A", "x"), q("B", q("A", "x"), q("C", "y"))) :: Nil
    }
  }
}
